﻿using JewelryStoreContracts.BindingModels;
using JewelryStoreContracts.ViewModels;
using JewerlyStoreDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace JewerlyStoreFileImplement.Models
{
    public class Shop : IShopModel
    {
        public int Id { get; private set; }
        public string ShopName { get; private set; } = string.Empty;
        public string Address { get; private set; } = string.Empty;
        public int MaxCountJewels { get; private set; }
        public DateTime DateOpen { get; private set; }
        public Dictionary<int, int> Jewels { get; private set; } = new();
        private Dictionary<int, (IJewelModel, int)>? _shopJewels = null;

        public Dictionary<int, (IJewelModel, int)> ShopJewels
        {
            get
            {
                if (_shopJewels == null)
                {
                    var source = DataFileSingleton.GetInstance();
                    _shopJewels = Jewels.ToDictionary(
                        x => x.Key,
                        y => ((source.Jewels.FirstOrDefault(z => z.Id == y.Key) as IJewelModel)!, y.Value)
                    );
                }
                return _shopJewels;
            }
        }

        public static Shop? Create(ShopBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Shop()
            {
                Id = model.Id,
                ShopName = model.ShopName,
                Address = model.Address,
                MaxCountJewels = model.MaxCountJewels,
                DateOpen = model.DateOpen,
                Jewels = model.ShopJewels.ToDictionary(x => x.Key, x => x.Value.Item2)
            };
        }

        public static Shop? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new Shop()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                ShopName = element.Element("ShopName")!.Value,
                Address = element.Element("Address")!.Value,
                MaxCountJewels = Convert.ToInt32(element.Element("MaxCountJewels")!.Value),
                DateOpen = Convert.ToDateTime(element.Element("DateOpen")!.Value),
                Jewels = element.Element("ShopJewels")!.Elements("ShopJewel").ToDictionary(
                    x => Convert.ToInt32(x.Element("Key")?.Value),
                    x => Convert.ToInt32(x.Element("Value")?.Value)
                )
            };
        }


        public void Update(ShopBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            ShopName = model.ShopName;
            Address = model.Address;
            MaxCountJewels = model.MaxCountJewels;
            DateOpen = model.DateOpen;
            if (model.ShopJewels.Count > 0)
            {
                Jewels = model.ShopJewels.ToDictionary(x => x.Key, x => x.Value.Item2);
                _shopJewels = null;
            }
        }
        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            ShopName = ShopName,
            Address = Address,
            MaxCountJewels = MaxCountJewels,
            DateOpen = DateOpen,
            ShopJewels = ShopJewels,
        };

        public XElement GetXElement => new(
            "Shop",
             new XAttribute("Id", Id),
             new XElement("ShopName", ShopName),
             new XElement("Address", Address),
             new XElement("MaxCountJewels", MaxCountJewels),
             new XElement("DateOpening", DateOpen.ToString()),
             new XElement("ShopJewels", Jewels.Select(x =>
             new XElement("ShopJewel",
             new XElement("Key", x.Key),
             new XElement("Value", x.Value)))
            .ToArray()));
    }
}
