﻿using JewelryStoreContracts.BusinessLogicsContracts;
using JewelryStoreContracts.SearchModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace JewerlyStoreView
{
	public partial class FormAddJewel : Form
	{
		private readonly ILogger _logger;
		private readonly IJewelLogic _logicJewel;
		private readonly IShopLogic _logicShop;
		public FormAddJewel(ILogger<FormAddJewel> logger, IJewelLogic logicJewel, IShopLogic logicShop)
		{
			InitializeComponent();
			_logger = logger;
			_logicJewel = logicJewel;
			_logicShop = logicShop;
		}

		private void FormAddJewel_Load(object sender, EventArgs e)
		{
			_logger.LogInformation("Загрузка списка украшений для пополнения");
			try
			{
				var list = _logicJewel.ReadList(null);
				if (list != null)
				{
					comboBoxJewel.DisplayMember = "JewelName";
					comboBoxJewel.ValueMember = "Id";
					comboBoxJewel.DataSource = list;
					comboBoxJewel.SelectedItem = null;
				}
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка загрузки списка украшеней");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}

			_logger.LogInformation("Загрузка списка магазинов для пополнения");
			try
			{
				var list = _logicShop.ReadList(null);
				if (list != null)
				{
					comboBoxShop.DisplayMember = "ShopName";
					comboBoxShop.ValueMember = "Id";
					comboBoxShop.DataSource = list;
					comboBoxShop.SelectedItem = null;
				}
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка загрузки списка магазинов");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void ButtonSave_Click(object sender, EventArgs e)
		{
			if (string.IsNullOrEmpty(numericUpDownCount.Text))
			{
				MessageBox.Show("Заполните поле Количество", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			if (comboBoxJewel.SelectedValue == null)
			{
				MessageBox.Show("Выберите украшение", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			if (comboBoxShop.SelectedValue == null)
			{
				MessageBox.Show("Выберите магазин", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			_logger.LogInformation("Пополнение магазина");
			try
			{
				var operationResult = _logicShop.AddJewel(new ShopSearchModel
				{
					Id = Convert.ToInt32(comboBoxShop.SelectedValue)
				},
				_logicJewel.ReadElement(new JewelSearchModel()
				{
					Id = Convert.ToInt32(comboBoxJewel.SelectedValue)
				})!, Convert.ToInt32(numericUpDownCount.Text));
				if (!operationResult)
				{
					throw new Exception("Ошибка при пополнении магазина. Дополнительная информация в логах.");
				}
				MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
				DialogResult = DialogResult.OK;
				Close();
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка создания заказа");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void ButtonCancel_Click(object sender, EventArgs e)
		{
			DialogResult = DialogResult.Cancel;
			Close();
		}
	}
}
