﻿using JewelryStoreContracts.BindingModels;
using JewelryStoreContracts.BusinessLogicsContracts;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace JewerlyStoreView
{
    public partial class FormShopSell : Form
    {
        private readonly ILogger _logger;
        private readonly IJewelLogic _logicJewel;
        private readonly IShopLogic _logicShop;

        public FormShopSell(ILogger<FormShopSell> logger, IJewelLogic logicJewel, IShopLogic logicShop)
        {
            InitializeComponent();
            _logger = logger;
            _logicJewel = logicJewel;
            _logicShop = logicShop;
        }

        private void FormShopSell_Load(object sender, EventArgs e)
        {
            _logger.LogInformation("Загрузка изделий для продажи");
            try
            {
                var list = _logicJewel.ReadList(null);
                if (list != null)
                {
                    comboBoxJewel.DisplayMember = "JewelName";
                    comboBoxJewel.ValueMember = "Id";
                    comboBoxJewel.DataSource = list;
                    comboBoxJewel.SelectedItem = null;
                }

            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки списка изделий");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonSell_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxCount.Text))
            {
                MessageBox.Show("Заполните поле Количество", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (comboBoxJewel.SelectedValue == null)
            {
                MessageBox.Show("Выберите изделий", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _logger.LogInformation("Создание продажи");
            try
            {
                var operationResult = _logicShop.SellJewels(
                    new JewelBindingModel
                    {
                        Id = Convert.ToInt32(comboBoxJewel.SelectedValue)
                    },
                    Convert.ToInt32(textBoxCount.Text)
                );
                if (!operationResult)
                {
                    throw new Exception("Ошибка при создании продажи. Дополнительная информация в логах.");
                }
                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания продажи");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
